###############################################################
# May 9 2011
# Jeff Goldsmith
#
# this file conducts a single simulation with 2 dimensional 
# predictors and the coefficient image that is the combination 
# of bivariate normal density functions. it is intended as 
# an example of the method on a simulated dataset.
###############################################################

rm(list=ls())

library(rgl)
library(mvtnorm)

setwd("~SET THE WD")
load("PCs2D.RDA")
source("plotImg2D.R")

set.seed(151)

# set some constants
I=100        # number of subjects
ONE = matrix(1, nrow=I, ncol=1)

## set some parameters
N=50
segs=(1:N)/N
var.ratio = (3);   sigeps=15; sigb = .6
#var.ratio = (1);   sigeps=20;	sigb = .6
#var.ratio = (1/3);   sigeps=50; sigb = .6

# for the Gibbs sampler
N.Iter = 250
N.BurnIn = 100

# hyperparameters
SigAlpha=10
a=-1.5
b=.75

# define true beta image
trueBeta = (.08*matrix(dmvnorm(cbind(rep(segs, each=N), 
    rep(sort(segs, decreasing=TRUE), N)), mean=c(.2, .3), 
    sigma=diag(c(.0025, .0015), 2)), 
    nrow=N, ncol=N) -
  .05*matrix(dmvnorm(cbind(rep(segs, each=N), 
    rep(sort(segs, decreasing=TRUE), N)), mean=c(.4, .8), 
    sigma=matrix(c(.002, -.001, -.001, .002),nrow=2, ncol=2)), 
    nrow=N, ncol=N) )
trueBeta = replace(trueBeta, which(abs(trueBeta) <.25), 0)

doGenData = TRUE
doIsingCAR = TRUE
doPlot = TRUE

#############################################################
#############################################################
#############################################################

if(doGenData){
  imgs=array(0, dim = c(N, N, I))

  for(k in 1:I){
    imgs[,,k] = meanImg
    for(j in 1:50){
      imgs[,,k] = imgs[,,k] + eigenImgs[,,j] * rnorm(1, 0, sqrt(load.var[j]))
    }
  }

  ## generate outcomes from true Beta function and functions measured without error
  Y <- sapply(1:I, function(u) sum(imgs[,,u]*trueBeta))
  Y=Y+rnorm(I, 0, sqrt(var(Y)/var.ratio))
  
  ## demean the images
  cur.meanImg=apply(imgs, c(1,2), mean)
  
  X=array(0, dim = c(N, N, I))
  for(k in 1:I){
    X[,,k]= imgs[,,k]-cur.meanImg  
  }
}

#############################################################
#############################################################
#############################################################

if(doIsingCAR){
  
  sigeps.inv = 1/sigeps
  sigbeta.inv = 1/sigb
  
  BetaHats=array(0, dim=c(N, N, N.Iter))
  Alpha=vector("numeric", length = N.Iter)
  Z.Ising=array(0, dim=c(N, N, N.Iter))
    
  BetaHats[,,1] = matrix(0, N, N)
  Z.Ising[,,1] = matrix(0, N, N)
  Alpha[1]=mean(Y)
  
  InProd=matrix(0, nrow=N, ncol=N)
  for(i in 1:N){
    for(j in 1:N){
      Xl=X[i,j,]
      InProd[i,j]=t(Xl)%*%(Xl)  
    }
  }

  xbeta = rep(0, I)
  tY=t(Y)

  time.start = proc.time()
  for(i.iter in 2:N.Iter){
  	    
    betaHat=BetaHats[,,i.iter-1]
    alpha = Alpha[i.iter-1]
    z = Z.Ising[,,i.iter-1]

    # update gamma and z
    for(j in 1:N){
      for(k in 1:N){
        delta = cbind(c(j-1, j, j+1, j), c(k, k-1, k, k+1))
        delta = replace(delta, which(delta<=0 | delta >N), NA)
		neighbors = sum(complete.cases(delta))
        
        Xl=X[j,k,]

        xbeta0 = xbeta - betaHat[j,k]*Xl;  txbeta0 = t(xbeta0)

        betabar =  mean(betaHat[delta], na.rm=TRUE)
        Sigl = 1/(sigeps.inv * InProd[j,k] + neighbors*sigbeta.inv)
        Mul = Sigl * ( sigeps.inv * (tY - alpha - txbeta0)%*%Xl + neighbors*sigbeta.inv * betabar   )
        betaStar = rnorm(1, Mul, sqrt(Sigl))
                
        xbeta1 = xbeta0+Xl*betaStar; txbeta1=t(xbeta1)

        IND = sum(1-2*z[delta], na.rm=TRUE)

        g = sqrt(2*pi*Sigl)*exp((-.5*sigeps.inv)*(txbeta0%*%xbeta0 + 2*(txbeta1-txbeta0)%*%(Y+alpha) - txbeta1%*%xbeta1) 
        	+ .5*(1/Sigl)*(betaStar-Mul)^2
            - a + b*IND )

        ppos = 1/(1+g)
      
        if(rbinom(1, 1, ppos)==1) {z[j,k] = 1; betaHat[j,k] = betaStar}
        else {z[j,k] = betaHat[j,k] = 0}

        xbeta = xbeta0 + betaHat[j,k]*Xl
      }  
    }

    BetaHats[,,i.iter]=betaHat

    Z.Ising[,,i.iter] = z
    
    # update alpha
    Sigma = solve(sigeps.inv * t(ONE)%*%(ONE) + (1/SigAlpha))
    mu.alpha = Sigma * ( sigeps.inv * (t(Y) - t(xbeta))%*%ONE )
    Alpha[i.iter] = rnorm(1, mu.alpha, sd=sqrt(Sigma))
 
# 
# plot each iteration as it happens:    
#    plotImg3d(segs, segs, trueBeta, size=7, col="red", labels = FALSE)
#    pointsImg3d(segs, segs, betaHat, size=7)
#

  }
  time.end = proc.time()
  
  betaHatIsing=apply(BetaHats[,,N.BurnIn:N.Iter], c(1,2), mean)
  
  plotImg3d(segs, segs, trueBeta, size=7, col="red", labels = FALSE)
  pointsImg3d(segs, segs, betaHatIsing, size=7)
  
  MSE.mrf.off = mean(betaHatIsing[which(trueBeta==0)]^2)
  MSE.mrf.on = mean((betaHatIsing[which(trueBeta!=0)]-trueBeta[which(trueBeta!=0)])^2)

}


#############################################################
#############################################################
#############################################################

if(doPlot){
  plotImg3d(segs, segs, trueBeta, size=7, labels = FALSE)
  pointsImg3d(segs, segs, betaHatIsing, size=7, col="red")

  c(MSE.mrf.off, MSE.mfr.on)
}


#############################################################
#############################################################
#############################################################
#############################################################
#############################################################
#############################################################